/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.swing.rule;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

import nu.zoom.catonine.stylerule.StyleRule;
import nu.zoom.swing.action.AbstractTypedAction;
import nu.zoom.swing.field.PatternField;
import nu.zoom.swing.layout.VerticalPanel;
import nu.zoom.ui.Resources;
import nu.zoom.ui.Resources.ResourceNotFoundException;

/**
 * A panel that allows editing of the styling rules in a collection of style
 * rules. Adding/removing style rules and defining the regular expression and
 * color of the log entries.
 * 
 * @author "Johan Maasing" &lt;johan@zoom.nu&gt;
 * 
 */
public class StyleRuleEditPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JCheckBox supressCheckBox;
	private JButton foregroundColorChooser;
	private JButton backgroundColorChooser;
	private StyleRulePatternField patternTextField;
	private StyleRule styleRule;

	public StyleRuleEditPanel(Resources messages) throws ResourceNotFoundException {
		super(new BorderLayout());
		ForegroundAction foregroundAction = new ForegroundAction(messages);
		BackgroundAction backgroundAction = new BackgroundAction(messages);
		foregroundColorChooser = new JButton(foregroundAction);
		backgroundColorChooser = new JButton(backgroundAction);
		JLabel patternLabel = new JLabel(messages.getMessage("nu.zoom.catonine.stylerule.pattern"));
		patternTextField = new StyleRulePatternField(patternLabel);
		patternTextField.setToolTipText(messages.getMessage("nu.zoom.catonine.stylerule.pattern.tt"));
		supressCheckBox = new JCheckBox(new SupressAction());

		JPanel colorPanel = new JPanel(new FlowLayout());
		colorPanel.add(foregroundColorChooser);
		colorPanel.add(backgroundColorChooser);

		VerticalPanel mainPanel = new VerticalPanel();
		mainPanel.addRow(patternLabel, patternTextField);
		mainPanel.addRow(messages.getMessage("nu.zoom.catonine.stylerule.supress"), supressCheckBox);
		mainPanel.addRow(colorPanel);
		add(mainPanel, BorderLayout.CENTER);
	}

	class ForegroundAction extends AbstractTypedAction {

		private static final long serialVersionUID = 1L;
		private final String chooserTooltip;

		public ForegroundAction(Resources messages) throws ResourceNotFoundException {
			super();
			setName(messages.getMessage("nu.zoom.catonine.stylerule.foreground"));
			setToolTip(messages.getMessage("nu.zoom.catonine.stylerule.foreground.tt"));
			this.chooserTooltip = messages.getMessage("nu.zoom.catonine.stylerule.foreground.tt");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			Color selectedColor =
					JColorChooser.showDialog(StyleRuleEditPanel.this, chooserTooltip,
							(styleRule != null) ? styleRule.getForeground() : Color.BLACK);
			if ((selectedColor != null) && (styleRule != null)) {
				styleRule.setForeground(selectedColor);
			}
		}
	}

	class BackgroundAction extends AbstractTypedAction {

		private static final long serialVersionUID = 1L;
		private final String chooserTooltip;

		public BackgroundAction(Resources messages) throws ResourceNotFoundException {
			super();
			setName(messages.getMessage("nu.zoom.catonine.stylerule.background"));
			setToolTip(messages.getMessage("nu.zoom.catonine.stylerule.background.tt"));
			this.chooserTooltip = messages.getMessage("nu.zoom.catonine.stylerule.foreground.tt");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			Color selectedColor =
					JColorChooser.showDialog(StyleRuleEditPanel.this, chooserTooltip,
							(styleRule != null) ? styleRule.getBackground() : Color.WHITE);
			if ((selectedColor != null) && (styleRule != null)) {
				styleRule.setBackground(selectedColor);
			}
		}
	}

	class SupressAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			boolean supress = supressCheckBox.isSelected();
			foregroundColorChooser.setEnabled(!supress);
			backgroundColorChooser.setEnabled(!supress);
			patternTextField.setEnabled(!supress);
			if (styleRule != null) {
				styleRule.setSupress(supress);
			}
		}
	}

	/**
	 * @param styleRule
	 *            The styleRule to set.
	 */
	public void setStyleRule(StyleRule styleRule) {
		if (styleRule == null) {
			unsetStyleRule();
		} else {
			this.styleRule = styleRule;
			final boolean supress = styleRule.isSupress();
			foregroundColorChooser.setEnabled(!supress);
			backgroundColorChooser.setEnabled(!supress);
			patternTextField.setEnabled(!supress);
			supressCheckBox.setEnabled(true);
			supressCheckBox.setSelected(supress);
			patternTextField.setText(styleRule.getPattern().toString());
		}
	}

	public void unsetStyleRule() {
		this.styleRule = null;
		patternTextField.setText("");
		foregroundColorChooser.setEnabled(true);
		backgroundColorChooser.setEnabled(true);
		patternTextField.setEnabled(true);
		supressCheckBox.setEnabled(true);
		supressCheckBox.setSelected(false);
	}

	class StyleRulePatternField extends PatternField {

		private static final long serialVersionUID = 1L;

		public StyleRulePatternField(JLabel label) {
			super(label);
		}

		@Override
		protected void validationPassed() {
			super.validationPassed();
			// Is there a selected style rule? Document might change even if
			// there is no active style rule (for example from "unset".
			if (styleRule != null) {
				styleRule.setPattern(getText());
			}
		}
	}
}
